(* +------------------------------------------------------------------------+
   |                                                                        |
   |                        Enregistrement des images                       |
   |                                                                        |
   +------------------------------------------------------------------------+ *)

open Unix
open Printf
open Graphics
open Global
open Widgets

(* liste des convertisseurs *)
let formats = [
  "ppm", "cat";
  "bmp", "ppmtobmp";
  "gif", "ppmtogif";
  "pcx", "ppmtopcx";
  "pgm", "ppmtopgm";
  "tga", "ppmtotga";
  "jpg", "cjpeg"
]

(* affiche un massage d'erreur *)
let erreur fen msg pixmap =
  let b = bouton_texte({fen with fx=fen.fx+fen.fw - owidth; fw=owidth},"ok") in
  fill_fentre fen fond_actif;
  set_color crayon;
  moveto (fen.fx+2) (fen.fy+2);
  draw_string msg;
  draw_bouton false b;
  let encore = ref(true) in
  while !encore do
    match vnement false {boutons=[b]; onglets=[]; entres=[]} with
      | Some(BOUTON "ok") -> encore := false
      | _ -> ()
  done;
  draw_image pixmap fen.fx fen.fy


(* demande un nom de fichier et enregistre l'image *)
let enregistre data pixmap =
  let px = Widgets.get_image data.fmsg in
  let entre = {ef=data.fmsg; leg="fichier: "; texte=data.fname} in

  match ask(entre) with
    | None -> ()
    | Some(nom) -> try
	data.fname <- nom;
	let l  = String.length nom in
	let i  = ref(l)            in
	while !i > 0 & nom.[!i-1] <> '.' do decr i done;
	let cmd   = List.assoc (String.sub nom !i (l - !i)) formats in
	let chout = open_process_out (sprintf "%s > %s" cmd nom)    in
	let t = dump_image pixmap in
	let w = Array.length t.(0) and h = Array.length t in
	fprintf chout "P6 %d %d 255\n" w h;
	for i=0 to h-1 do for j=0 to w-1 do
	  let c = t.(i).(j) in
	  output_byte chout ((c lsr 16) land 0xff);
	  output_byte chout ((c lsr  8) land 0xff);
	  output_byte chout ( c         land 0xff)
	done done;
	let _ = close_process_out chout in
	draw_image px data.fmsg.fx data.fmsg.fy
      with
	| Not_found         -> erreur entre.ef "format inconnu"  px
	| Sys_error(e)      -> erreur entre.ef e                 px
	| Unix_error(e,_,_) -> erreur entre.ef (error_message e) px
	| _                 -> erreur entre.ef "erreur systme"  px
	      
	
